@@ -34,6 +34,8 @@ const (
 	FilterFlagName             = "filter"
 	FilterAffectedFlagName     = "filter-affected"
 	FilterAllowDestroyFlagName = "filter-allow-destroy"
+	FilterFileFlagName         = "filters-file"
+	NoFilterFileFlagName       = "no-filters-file"
 
 	// Scaffolding related flags.
 	RootFileNameFlagName  = "root-file-name"
@@ -249,15 +251,7 @@ func NewFilterFlags(l log.Logger, opts *options.TerragruntOptions) cli.Flags {
 						l.Warnf("Warning: You have uncommitted changes. The --filter-affected flag may not include all your local modifications.")
 					}
 
-					defaultBranch := "main"
-
-					if b, err := gitRunner.Config(ctx.Context, "init.defaultBranch"); err == nil && b != "" {
-						l.Debugf("Using default branch discovered from git config: %s", b)
-
-						defaultBranch = b
-					} else {
-						l.Warnf("Failed to get default branch from `git config init.defaultBranch`, using main.")
-					}
+					defaultBranch := gitRunner.GetDefaultBranch(ctx.Context, l)
 
 					opts.FilterQueries = append(opts.FilterQueries, fmt.Sprintf("[%s...HEAD]", defaultBranch))
 
@@ -280,6 +274,40 @@ func NewFilterFlags(l log.Logger, opts *options.TerragruntOptions) cli.Flags {
 				},
 			},
 		),
+		flags.NewFlag(
+			&cli.GenericFlag[string]{
+				Name:        FilterFileFlagName,
+				EnvVars:     tgPrefix.EnvVars(FilterFileFlagName),
+				Destination: &opts.FiltersFile,
+				Usage:       "Path to a file containing filter queries, one per line. Default is .terragrunt-filters. Requires the 'filter-flag' experiment.",
+				Action: func(_ *cli.Context, val string) error {
+					// Check if the filter-flag experiment is enabled
+					if !opts.Experiments.Evaluate("filter-flag") {
+						return cli.NewExitError("the --filters-file flag requires the 'filter-flag' experiment to be enabled. Use --experiment=filter-flag or --experiment-mode to enable it", cli.ExitCodeGeneralError)
+					}
+					return nil
+				},
+			},
+		),
+		flags.NewFlag(
+			&cli.BoolFlag{
+				Name:        NoFilterFileFlagName,
+				EnvVars:     tgPrefix.EnvVars(NoFilterFileFlagName),
+				Destination: &opts.NoFiltersFile,
+				Usage:       "Disable automatic reading of .terragrunt-filters file. Requires the 'filter-flag' experiment.",
+				Action: func(_ *cli.Context, val bool) error {
+					if !val {
+						return nil
+					}
+
+					// Check if the filter-flag experiment is enabled
+					if !opts.Experiments.Evaluate("filter-flag") {
+						return cli.NewExitError("the --no-filters-file flag requires the 'filter-flag' experiment to be enabled. Use --experiment=filter-flag or --experiment-mode to enable it", cli.ExitCodeGeneralError)
+					}
+					return nil
+				},
+			},
+		),
 	}
 }
 
