@@ -36,6 +36,7 @@ type Component interface {
 	Kind() Kind
 	Path() string
 	SetPath(string)
+	DisplayPath() string
 	External() bool
 	SetExternal()
 	Reading() []string
@@ -64,6 +65,9 @@ type Component interface {
 // because it can help us determine how the
 // Component should be run or enqueued later.
 type DiscoveryContext struct {
+	WorkingDir string
+	Ref        string
+
 	Cmd  string
 	Args []string
 }
