@@ -171,6 +171,27 @@ func resolveUnitsFromDiscovery(
 			return nil, err
 		}
 
+		// If --source is provided, compute the per-unit source by combining the base source
+		// with the path from the unit's terragrunt.hcl source configuration
+		if stack.Execution != nil && stack.Execution.TerragruntOptions != nil &&
+			stack.Execution.TerragruntOptions.Source != "" {
+			unitConfig := unit.Config()
+			if unitConfig != nil {
+				unitSource, sourceErr := config.GetTerragruntSourceForModule(
+					stack.Execution.TerragruntOptions.Source,
+					canonicalConfigPath,
+					unitConfig,
+				)
+				if sourceErr != nil {
+					return nil, tgerrors.Errorf("failed to compute source for unit %s: %w", unit.DisplayPath(), sourceErr)
+				}
+
+				if unitSource != "" {
+					unitOpts.Source = unitSource
+				}
+			}
+		}
+
 		// Skip units without Terraform configuration
 		skip, err := shouldSkipUnitWithoutTerraform(unit, canonicalDir, unitLogger)
 		if err != nil {
@@ -198,15 +219,37 @@ func resolveUnitsFromDiscovery(
 		units = append(units, unit)
 	}
 
-	// Ensure the stack-level default cache directory exists so tests that expect the root cache can find it,
-	// even when per-unit download directories are used.
-	if stackDefaultDownloadDir != "" {
-		_ = os.MkdirAll(stackDefaultDownloadDir, os.ModePerm) // best-effort
-	}
-
 	return units, nil
 }
 
+// checkLocalStateWithGitRefs checks if any unit has a Git ref in its discovery context
+// but no remote state configuration, and logs a warning if so.
+func checkLocalStateWithGitRefs(l log.Logger, units []*component.Unit) {
+	for _, unit := range units {
+		discoveryCtx := unit.DiscoveryContext()
+		if discoveryCtx == nil {
+			continue
+		}
+
+		if discoveryCtx.Ref == "" {
+			continue
+		}
+
+		unitConfig := unit.Config()
+		if unitConfig == nil {
+			continue
+		}
+
+		if unitConfig.RemoteState == nil || unitConfig.RemoteState.BackendName == "local" {
+			l.Warnf(
+				"One or more units discovered using Git-based filter expressions (e.g. [HEAD~1...HEAD]) do not have a remote_state configuration. This may result in unexpected outcomes, such as outputs for dependencies returning empty. It is strongly recommended to use remote state when working with Git-based filter expressions.",
+			)
+
+			return
+		}
+	}
+}
+
 // NewRunnerPoolStack creates a new stack from discovered units.
 func NewRunnerPoolStack(
 	ctx context.Context,
@@ -267,13 +310,16 @@ func NewRunnerPoolStack(
 		return nil, err
 	}
 
+	// Check for units with Git refs but no remote state configuration
+	checkLocalStateWithGitRefs(l, units)
+
 	// Record exclude-dir reasons in report before filtering.
 	if runner.Stack.Execution != nil && runner.Stack.Execution.Report != nil && len(terragruntOptions.ExcludeDirs) > 0 {
 		for _, unit := range units {
 			for _, dir := range terragruntOptions.ExcludeDirs {
 				cleanDir := dir
 				if !filepath.IsAbs(cleanDir) {
-					cleanDir = util.JoinPath(terragruntOptions.WorkingDir, cleanDir)
+					cleanDir = filepath.Join(terragruntOptions.WorkingDir, cleanDir)
 				}
 
 				cleanDir = util.CleanPath(cleanDir)
@@ -907,10 +953,6 @@ func FilterDiscoveredUnits(discovered component.Components, units []*component.U
 // WithOptions updates the stack with the provided options.
 func (r *Runner) WithOptions(opts ...common.Option) *Runner {
 	for _, opt := range opts {
-		if opt == nil {
-			continue
-		}
-
 		opt.Apply(r)
 	}
 
